/*
 * ChainedReader.java
 *
 * 作成日: 2012/01/13
 * Copyright(C)2008 Kyushu Institute of Technology., All right reserved. 
 */
package jp.ac.kyutech.SRP;

import java.io.IOException;
import java.io.Reader;
import java.util.LinkedList;

/**
 *
 */
public class ChainedReader extends Reader {

    protected LinkedList<Reader> readers = new LinkedList<Reader>();

    private Reader current;

    /**
     * デフォルトコンストラクタです。
     */
    public ChainedReader() {
        addReader(new BlankReader());
    }

    /**
     * @param first
     */
    public ChainedReader(Reader first) {
        addReader(first);
    }

    /**
     * @param readers
     */
    public ChainedReader(Reader[] readers) {
        for (Reader reader : readers) {
            addReader(reader);
        }
    }

    /**
     * @param reader
     * @return
     */
    public ChainedReader addReader(Reader reader) {
        readers.addLast(reader);
        if (current == null) {
            current = readers.removeFirst();
        }
        return this;
    }

    @Override
    public int read(char[] cbuf, int off, int len) throws IOException {
        int bit = current.read(cbuf, off, len);
        while ((bit == -1 || bit != len) && readers.size() > 0) {
            try {
                current.close();
            }
            catch (IOException e) {
                e.printStackTrace();
            }
            current = readers.removeFirst();
            bit += current.read(cbuf, bit, (len - bit));
        }
        return bit;
    }

    @Override
    public int read() throws IOException {
        int bit = current.read();
        if (bit == -1 && readers.size() > 0) {
            try {
                current.close();
            }
            catch (IOException e) {
                e.printStackTrace();
            }
            current = readers.removeFirst();
            bit = read();
        }
        return bit;
    }

    @Override
    public void close() throws IOException {
        current.close();
    }

    @Override
    public boolean markSupported() {
        return current.markSupported();
    }

    @Override
    public synchronized void mark(int i) throws IOException {
        current.mark(i);
    }

    @Override
    public synchronized void reset() throws IOException {
        current.reset();
    }

    @Override
    public long skip(long l) throws IOException {
        return current.skip(l);
    }

    class BlankReader extends Reader {
        @Override
        public void close() throws IOException {
        }
        @Override
        public int read(char[] cbuf, int off, int len) throws IOException {
            return -1;
        }
    }
}
